-- Ausgangspunkt: Als Benutzer SCOTT ausführen
-- SCOTT benötigt execute-Recht an CTX_DDL

-- Einfacher Full-Text-Index
-- Tabelle zur Indizierung
create table full_text(
	id integer,
	datei_name varchar2(50 char),
	text blob); 

-- Erzeugung des Index
create index idx_full_text_ctx on full_text(text) 
  indextype is ctxsys.context;

-- Abfrage gegen die Tabelle mit CONTAINS
select score(1), datei_name, text 
  from full_text 
 where contains(text, 'RATIO', 1) > 0;

-- Index aktualisieren
call ctx_ddl.sync_index('IDX_FULL_TEXT_CTX');

-- Blick unter die Haube: Darstellung von Indextabellen
select token_text, token_type type, token_last last, 
       token_count count, token_info
  from dr$idx_full_text_ctx$i 
 where token_text like 'Ab%' 
   and rownum < 15;
   
-- Parametrierung des Text-Indexes mit einem Preference-Objekt:
begin 
  ctx_ddl.create_preference('GERMAN_LEXER', 'BASIC_LEXER'); 
  ctx_ddl.set_attribute('GERMAN_LEXER', 'ALTERNATE_SPELLING', 'GERMAN'); 
  ctx_ddl.set_attribute('GERMAN_LEXER', 'COMPOSITE', 'GERMAN'); 
  ctx_ddl.set_attribute('GERMAN_LEXER', 'MIXED_CASE', 'NO'); 
  ctx_ddl.set_attribute('GERMAN_LEXER', 'BASE_LETTER', 'YES'); 
  ctx_ddl.set_attribute('GERMAN_LEXER', 'INDEX_STEMS', 'GERMAN'); 
end;
/

-- Erstellung eines Index mit Referenz auf das Preference-Objekt:
drop index idx_full_text;
create index_idx_full_text on full_text(text) 
  indextype is ctxsys.context
  parameters ('LEXER german_lexer');


-- Verwendung eines Textindexes zur Suche in mehreren Textspalten

-- Die Erstellung der Prozedur basiert auf dieser View:
create or replace view emp_vw as
select e.rowid row_id, e.ename, e.job, d.dname, d.loc, s.grade
  from emp e
  join dept d on e.deptno = d.deptno
  join salgrade s on e.sal between s.losal and s.hisal;


-- Schritt 1: Prozedur mit diesem Footprint erstellen, die mehrere Spaltenwerte
-- in einem strukturierten Text zusammenfasst. Kann XML sein, muss aber nicht
create or replace procedure indiziere_spalten(
  rid in rowid,
  tlob in out nocopy clob)
as
begin
  select xmlelement("Mitarbeiter",
           xmlforest(
             ename "Name",
             job "Beruf",
             dname "Abteilung",
             loc "Ort",
             grade "Gehaltsstufe"
           )
         ).getClobVal()
    into tlob
    from emp_vw 
   where row_id = rid;
end;
/

-- Schritt 2: Preference-Objekt erstellen, dass dem Index später erklärt,
-- wie er Daten für den Index gewinnen kann. Optionen: Es können auch deutsche
-- Wortstammregeln etc. mit übergeben werden, hierzu ein Lexer-Objekt erstellen.
begin
  -- Datastore: Wo sind meine Daten?
  ctx_ddl.create_preference('MITARBEITER_UDS', 'USER_DATASTORE');
  ctx_ddl.set_attribute('MITARBEITER_UDS', 'PROCEDURE', 'INDIZIERE_SPALTEN');

  -- Lexer: Wie soll indiziert werden?
  ctx_ddl.create_preference('GERMAN_LEXER', 'BASIC_LEXER');
  ctx_ddl.set_attribute('GERMAN_LEXER', 'ALTERNATE_SPELLING', 'GERMAN');
  ctx_ddl.set_attribute('GERMAN_LEXER', 'COMPOSITE', 'GERMAN');
  ctx_ddl.set_attribute('GERMAN_LEXER', 'MIXED_CASE', 'NO');
  ctx_ddl.set_attribute('GERMAN_LEXER', 'BASE_LETTER', 'YES');
  ctx_ddl.set_attribute('GERMAN_LEXER', 'INDEX_STEMS', 'GERMAN');
end;
/

-- Schritt 3: Text-Index erstellen, Preference-Objekte übergeben
-- Indizierte Spalte muss varchar2 sein, ist aber egal, welche, nur aus Tabelle EMP
create index idx_emp_full on emp(ename)
indextype is ctxsys.context
parameters ('DATASTORE mitarbeiter_uds LEXER german_lexer');


-- Enjoy! Contains-Abfrage mit AND, LIKE und Case-Insensitive
select ename, job, dname, loc, grade
  from emp_vw
 where contains(ename, 'accounting and man%') > 0;
 
