-- Datenmodellierung bersetzbarer Datenbankanwendungen
-- Liste der von Oracle untersttzten Sprachen
select value
  from v$nls_valid_values
 where parameter = 'LANGUAGE'
   and isdeprecated = 'FALSE';

-- Aktuell eingestellte Sprache
select value
  from nls_session_parameters
 where parameter = 'NLS_LANGUAGE';

-- Anlegen einer eigenen Tabelle fr Sprachuntersttzung
create table languages (
  language_id number primary key,
  oracle_language varchar2(30 char) not null,
  iso_3166_A2_code char(2 byte) not null,
  iso_3166_A3_code char(3 byte) not null,
  rfc_1766_code varchar2(24 char) not null,
  display_value varchar2(64 char) not null,
  sort_order number(3,0),
  constraint languages_pk primary key (language_id));

insert into languages 
values (
  49, 'GERMAN', 'DE', 'DEU', 'de-DE', 'Deutsch', 10); 
insert into languages 
values (
  1, 'AMERICAN', 'US', 'USA', 'en-US', 'Amerikanisch', 0);

-- Mapping von Oracle-Sprachnamen auf Standardformen
select utl_i18n.map_language_from_iso('de-DE')
  from dual;


-- bersetzung von Stammdaten durch abgeleitete Primrschlssel
alter table dept
modify (deptno number(4,2));

select *
  from dept;

insert into dept
values (20.49, 'FORSCHUNG', 'DALLAS');

insert into dept
values (20.33, 'RECHERCHE', 'DALLAS');

commit;

-- Abfrage von Stammdaten in einer gegebenen Sprache
select deptno, dname, loc
  from (select trunc(deptno) deptno, dname, loc,
               rank() over (
                 partition by loc
                 order by deptno desc) rang
          from dept d 
    where mod(deptno, 1) in (0, 0.49)))
 where rang = 1;
 
-- Verallgemeinerung mit Sprachtabelle
create table languages as
select value language_code, rownum * 0.01 language_id
  from v$nls_valid_values
 where parameter = 'LANGUAGE'
   and isdeprecated = 'FALSE';

update dept
   set deptno = trunc(deptno) +
                (select language_id
                   from languages
                  where language_code = 'GERMAN')
 where mod(deptno, 1) = 0.49;

commit;

-- Kapselung durch eine VIEW
create or replace view get_language_id as
select language_id
  from languages l
  join nls_session_parameters p
    on l.language_code = p.value
 where p.parameter = 'NLS_LANGUAGE';

create or replace view dept_v as
select deptno, dname, loc
  from (select trunc(deptno) deptno, dname, loc,
                rank() over (partition by loc
                              order by deptno desc) rang
          from dept d
          join get_language_id l
            on mod(d.deptno, 1) in (0, l.language_id))
 where rang = 1;

-- Test
alter session set nls_language = 'FRENCH';

select *
  from dept_v
 where deptno = 20;

alter session set nls_language = 'GERMAN';

select *
  from dept_v
 where deptno = 20;

 
-- bersetzungstabelle mit virtuellem Primrschlssel
-- Tabelle mit Sprache
create table salutation(
  sal_id varchar2(10 char) not null,
  sal_language varchar2(30 byte),
  sal_name varchar2(50 char));

-- Erster Versuch eines konditionalen Primrschlssels
create unique index idx_salutation_u on salutation(
  case sal_language when 'GERMAN' then sal_id end);

-- Ohne Constraint ist eine Fremdschlsselbeziehung nicht mglich:
create table person(
  per_id number,
  per_sal_id varchar2(10 char),
  per_name varchar2(20 char),
  constraint pk_person primary key(per_id),
  constraint fk_per_sal_id foreign key(per_sal_id)
    references salutation(sal_id)
);

-- Lsung ber virtuelle Spalten
-- Start mit einer Sprachentabelle
create table translation_language(
  trl_id varchar2(30 byte),
  trl_sort_seq number,
  constraint pk_trans_lang primary key(trl_id)
) organization index;

-- Anlage der SALUTATION-Tabelle mit virtueller Spalten
create table salutation(
  sal_id varchar2(10 char),
  sal_trl_id varchar2(30 byte),
  sal_name varchar2(50 char),
  sal_id_unique varchar2(10 char)
    as (case sal_trl_id when 'GERMAN' then sal_id end) virtual,
  constraint u_salutation unique(sal_id_unique),
  constraint fk_sal_trl_id foreign key(sal_trl_id)
    references translation_language(trl_id)
);

-- Referenz auf die Tabelle funktioniert
create table person(
  per_id number,
  per_sal_id varchar2(10 char),
  per_name varchar2(20 char),
  constraint pk_person primary key(per_id),
  constraint fk_per_sal_id foreign key(per_sal_id)
    references salutation(sal_id_unique)
);

-- Test:
-- Testdaten einfgen
insert into translation_language(trl_id, trl_sort_seq)
select 'GERMAN', 0 from dual union all
select 'AMERICAN', 10 from dual;

insert into salutation(sal_id, sal_trl_id, sal_name)
select 'HR', 'GERMAN', 'Herr' from dual union all
select 'HR', 'AMERICAN', 'Mr.' from dual union all
select 'FR', 'GERMAN', 'Frau' from dual union all
select 'FR', 'AMERICAN', 'Mrs.' from dual;

insert into person(per_id, per_sal_id, per_name)
select 1, 'HR', 'Meyer' from dual union all
select 2, 'FR', 'Schmitz' from dual;

commit;

-- Abfrage
create or replace view person_vw as
select *
  from (select p.per_id,
               s.sal_name per_salutation,
               p.per_name,
               rank() over (
                 partition by p.per_id
                 order by l.trl_sort_seq desc) rang
          from person p
          join salutation s
            on p.per_sal_id = s.sal_id
          join translation_language l
            on s.sal_trl_id = l.trl_id
         where sys_context('USERENV', 'LANGUAGE') like l.trl_id || '%')
 where rang = 1;

select *
  from person_vw;

alter session set nls_language = 'AMERICAN';

select *
  from person_vw;

  
-- Anmerkung SYS_CONTEXT
select sys_context('USERENV', 'LANGUAGE')
  from dual;

select sys_context('USERENV', 'LANG')
  from dual;
